/*******************************************************************
File Commands.e
file with all the code for the commands required in the project
mirrorx/y rotate, file operations, etc.
*******************************************************************/

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <string.h>
#include <stdlib.h>
#include "piecewin.h"
#include "gridwin.h"
#include "commands.h"
Window load_window,save_window,mirror_x_window,mirror_y_window,quit_window,rotate_window;
Window save_files[10],new_file_window;
GC text_gc;
/*******************************************************************
new_file_setup(void)
set the grid up for the user to create a new puzzle to save to disk.
When the user wants to create a new puzzle we allow them to place
the pieces anywhere
*******************************************************************/
void new_file_setup(void)
	{
	extern struct play_grid grid;
	extern struct piece_rec piece_array[6];
	int i, j;
	for(i=0;i<MAXSIZE;i++)
		for(j=0;j<MAXSIZE;j++)
			{
			/* set all squares as wanted so user can use them */
			grid.matrix[i][j].wanted = true;
			grid.matrix[i][j].filled_by = 0;
			}
	for (i=1;i<6;i++)
		{
		/* set all pieces to unused initially */
		piece_array[i].in_use = false;
		display_piece_on_piecewin(i);
		}
        /* draw the grid with new settings */
	draw_grid();
	}
/*******************************************************************
file_operate(i,load,save,new_file)
performs the action of saving a file to disk/ reading a file from disk
as long as prerequisites are met.
*******************************************************************/
void file_operate(int i,enum boolean load,enum boolean save,enum boolean new_file)
	{
	extern Display *display;
	char fname[28] ;
	sprintf(fname,"grid%d",i);
	/* did they want to save the current design */
	if (save)
		/* is the current design a new one or a loaded one ? */
		if (new_file)
			{
			/* save the grid */
			save_grid(fname);
			/* report saving of grid */
			sprintf(fname,"Saved file %d",i);
			info(fname);
			return;
			}
		else
			{
			info("Cannot write unless new file created");
			return;
			}
	else if (load)
			{
			get_grid(fname);
			return;
			}
	/* they did not specify load or save first */
	info("need to 'save' or 'load' first dude !"); 
	XBell(display,100);
	}
/*******************************************************************
void mirror_y(void)
mirrors the current piece about an imaginary y axis. An array was not
used in this and other operations due to the simplistic requirements
ie. all operations only involve multiplication by -1

*******************************************************************/

void mirror_y(void)
	{
	extern struct piece_rec piece_array[6];
	extern int active_piece;
	int i;
	for (i=0;i<3;i++)
		{
		/* for each block */
		piece_array[active_piece].block[i].x *= -1;
		}
	/* redraw piece in new perspective */
	display_piece_on_piecewin(active_piece);
	return;
	}
/*******************************************************************
void rotate(void)
rotate the piece about a point for 180 degrees. An array was not used
due to the simplistic calculations invloved
*******************************************************************/

void rotate(void)
	{
	extern struct piece_rec piece_array[6];
	extern int active_piece;
	int i;
	for (i=0;i<3;i++)
		{
		piece_array[active_piece].block[i].y *= -1;
		piece_array[active_piece].block[i].x *= -1;
		}
	display_piece_on_piecewin(active_piece);
	return;
	}
/*******************************************************************
void mirror_x(void)
mirrors the current piece about an imaginary x axis.
*******************************************************************/
void mirror_x(void)
	{
	extern struct piece_rec piece_array[6];
	extern int active_piece;
	int i;
	for (i=0;i<3;i++)
		{
		piece_array[active_piece].block[i].y *= -1;
		}
	display_piece_on_piecewin(active_piece);
	return;
	}
/*******************************************************************
void init_commands()
initialise the command windows - location, input allowed and info
displayed on them
*******************************************************************/
void init_commands()
	{
	char nums[11];
	extern XSizeHints main_win_size;
	char title[20] ;
	int i;
	extern GC text_gc;
	extern unsigned long foreground,background;
	extern Display *display;
	extern Window main_window,rotate_window,load_window,save_window,quit_window,mirror_x_window,mirror_y_window;
	extern Window save_files[10],new_file;
	strcpy(nums,"1234567890");
	strcpy(title,"Amazing Block Game");
	load_window = XCreateSimpleWindow(display,main_window,
		20,(30 + MAXSIZE * GRID_SIZE),50,30,2,foreground,background);
	save_window = XCreateSimpleWindow(display,main_window,
		100,(30 + MAXSIZE * GRID_SIZE),50,30,2,foreground,background);
	quit_window = XCreateSimpleWindow(display,main_window,
		180,(30 + MAXSIZE * GRID_SIZE),50,30,2,foreground,background);
	mirror_x_window = XCreateSimpleWindow(display,main_window,
		260,(30 + MAXSIZE * GRID_SIZE),50,30,2,foreground,background);
        mirror_y_window = XCreateSimpleWindow(display,main_window,
		340,(30 + MAXSIZE * GRID_SIZE),50,30,2,foreground,background);
	rotate_window = XCreateSimpleWindow(display,main_window,
		420,(30 + MAXSIZE * GRID_SIZE),50,30,2,foreground,background);
	new_file_window = XCreateSimpleWindow(display,main_window,
		420,(MAXSIZE * GRID_SIZE)- 10,50,30,2,foreground,background);
	for (i=0;i<10;i++)
		save_files[i] =  XCreateSimpleWindow(display,main_window,
			20 + (i * 40),(180 + MAXSIZE * GRID_SIZE),30,30,2,foreground,background);
        DEBUG("windows created \n");
	XSetStandardProperties(display,main_window,title,title,None,
				None,None,&main_win_size);
	text_gc = XCreateGC(display,load_window,0,0);
	XSetBackground(display,text_gc,background);
	XSetForeground(display,text_gc,foreground);

	XSelectInput(display,load_window,ButtonPressMask | ExposureMask);
	XSelectInput(display,quit_window,ButtonPressMask| ExposureMask);
	XSelectInput(display,save_window,ButtonPressMask| ExposureMask);
        XSelectInput(display,mirror_x_window,ButtonPressMask| ExposureMask);
        XSelectInput(display,mirror_y_window,ButtonPressMask| ExposureMask);
        XSelectInput(display,rotate_window,ButtonPressMask| ExposureMask);
        XSelectInput(display,new_file_window,ButtonPressMask| ExposureMask);
 	for (i=0;i<10;i++)
	        XSelectInput(display,save_files[i],ButtonPressMask| ExposureMask);

	DEBUG("input selected \n");
	XMapWindow(display,quit_window);
	XMapWindow(display,load_window);
	XMapWindow(display,save_window);
	XMapWindow(display,mirror_x_window);
	XMapWindow(display,mirror_y_window);
	XMapWindow(display,rotate_window);
	XMapWindow(display,new_file_window);
	for (i=0;i<10;i++)
		XMapWindow(display,save_files[i]);

	DEBUG("command windows mapped\n");
	XDrawImageString(display,load_window,text_gc,5,5,"Load",4);
        XDrawImageString(display,save_window,text_gc,5,5,"Save",4);
	XDrawImageString(display,quit_window,text_gc,5,5,"Quit",4);
	XDrawImageString(display,mirror_x_window,text_gc,5,5,"X mirror",8);
	XDrawImageString(display,mirror_y_window,text_gc,5,5,"Y mirror",8);
	XDrawImageString(display,rotate_window,text_gc,5,5,"Rotate",6);
	XDrawImageString(display,new_file_window,text_gc,5,5,"New File",8);
	DEBUG("info on command windows\n");
	for (i=0;i<10;i++)
		XDrawImageString(display,save_files[i],text_gc,5,5,&nums[i],1);


	}
