/*******************************************************************
file gridwin.c
all operations involving the grid window are stored here
*******************************************************************/
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <stdio.h>
#include <math.h>
#include "piecewin.h"
#include "gridwin.h"
Window gridwindow;/* the window puzzles are made/solved on */
struct play_grid grid;/* structure containing puzzle info */
/*******************************************************************
void display_block_on_gridwin(int x, int y, int piece_num)
draws the block in the correct location on the gridwindow
*******************************************************************/

void display_block_on_gridwin(int x, int y, int piece_num)
	{
	extern int active_piece;
	extern Window gridwindow;
	int tempx,tempy,i,j;
	tempx = (x-1) * GRID_SIZE;
	tempy = (y-1)* GRID_SIZE;
	for (j=0;j<16;j++)
		for (i=0;i<8;i++)
			draw_pixel(tempx + j * 3,
				   tempy + i * 3,
				   gridwindow,
				   piece_num);
	}
/*******************************************************************
void display_piece_on_grid(int piece_num)
draws the piece in the correct location on the gridwindow
*******************************************************************/

void display_piece_on_grid(int piece_num)
	{
	extern struct piece_rec piece_array[6];
	extern Display *display;
	extern Window gridwindow;
	int xloc,yloc;
	int i;
	DEBUG("displaying piece\n");
	for (i=0;i<3;i++)
		{	
		xloc = piece_array[piece_num].block[i].x + piece_array[piece_num].location.x;
		yloc = piece_array[piece_num].block[i].y + piece_array[piece_num].location.y;
		/* clear area for the block to be drawn on */
		XClearArea(display,gridwindow,(xloc-1)*GRID_SIZE,(yloc-1)*GRID_SIZE,48,24,0);
		DEBUG("drawing a block\n");
		display_block_on_gridwin(xloc,yloc,piece_num);

		};
	}
/*******************************************************************
enum boolean can_place(int mousex,int mousey)
test to see if piece fits on grid by
	1) testing piece not already used
	2) testing each block can be put down.
*******************************************************************/
enum boolean can_place(int mousex,int mousey)
	{
	extern struct piece_rec piece_array[6];
	extern struct play_grid grid;
	extern int active_piece;
	int i;
	int x,y;

	if (piece_array[active_piece].in_use)
		return false;
	for (i=0;i<3;i++)
		{

		x = mousex + piece_array[active_piece].block[i].x;
		y = mousey + piece_array[active_piece].block[i].y;
       		
		if (((grid.matrix[x - 1*(abs(x)/x)][y-1*(abs(y)/y)].wanted) &&
			(!grid.matrix[x - 1*(abs(x)/x)][y-1*(abs(y)/y)].filled_by))
		&&  ((grid.matrix[x][y-1*(abs(y)/y)].wanted) &&
			(!grid.matrix[x][y-1*(abs(y)/y)].filled_by)))
			{
			/* no problem */
               	        }
		else 
			{
			info("Bad position for piece, overlap or not in wanted area");
			return false;
			}
                }
	return true;

	}
/*******************************************************************
void place_here(int mousex, int mousey)
place the active piece on the grid. the top left hand corner is always
placed where the mouse was clicked
*******************************************************************/
void place_here(int mousex, int mousey)
	{
	extern int active_piece;
	extern Display *display;
	extern Window piece_windows[5];
	extern struct piece_rec piece_array[6];
	extern struct play_grid grid;
        int i,x,y;
	for (i=0;i<3;i++)
		{
		/* get x,y position of block as an aggregate of where
		mouse was clicked and where the block is in reference to
		the piece*/
		x = mousex + piece_array[active_piece].block[i].x;
		y = mousey + piece_array[active_piece].block[i].y;
		/* the two squares occupied by the piece are tagged as
		being filled by this piece, the "abs() bits" are for
		when the piece is on the negative side of the x, y axis */
		grid.matrix[x - 1*(abs(x)/x)][y-1*(abs(y)/y)].filled_by = active_piece;
		grid.matrix[x][y-1*(abs(y)/y)].filled_by = active_piece;
		/* tag piece as used */
		piece_array[active_piece].in_use  = true;
		/* remember where we put it */
		piece_array[active_piece].location.x = mousex;
		piece_array[active_piece].location.y = mousey;

		}
	/* clear the window of the piece just placed
	to indicate we cannot use it any more */
	XClearWindow(display,piece_windows[active_piece]);
	/* select a new active piece if any not used */
	for (i=6;i>1;i--)
		if (!piece_array[i].in_use)
			active_piece = i;
	/* redraw grid with new piece on it */
	draw_grid();
	}

/*******************************************************************
void save_grid(char *filename)
	write the grid to file
*******************************************************************/
void save_grid(char *filename)
	{
	extern struct play_grid grid;
	FILE *fp;
	int i,j;

	fp = fopen(filename,"w");

	for (i=0;i<MAXSIZE;i++)
		for (j=0;j<MAXSIZE;j++)
			{
			if (grid.matrix[j][i].filled_by)
				fprintf(fp,"1 ");
			else
				fprintf(fp,"0 ");
        		}
        fclose(fp);
	}
/*******************************************************************
void get_grid(char *fname)
read grid from file
*******************************************************************/

void get_grid(char *fname)
	{
	FILE *fp;
	int i,j;
	int k;
	extern struct play_grid grid;
	extern struct piece_rec piece_array[6];
	if( ( fp = fopen(fname,"r") ) == NULL )
		{
			perror( fname );
			return;
		}
	DEBUG("reading in grid\n");
	for (i=0;i<MAXSIZE;i++)
		for (j=0;j<MAXSIZE;j++)
			{
			fscanf(fp,"%d",&k);
			if (k)
				grid.matrix[j][i].wanted = true;
			else
				grid.matrix[j][i].wanted = false;

			grid.matrix[i][j].filled_by = 0;
			}
        DEBUG("finished reading in file\n");
	fclose(fp);
	/* initialise pieces for use */
	for (i=1;i<6;i++)
		{
		piece_array[i].in_use = false;
	        display_piece_on_piecewin(i);      
		}
	/* display new puzzle */
	draw_grid();
	return;
	}
/*******************************************************************
void draw_wanted_square(int x,int y)
draws a shaded square in appropiate location to indicate where we can
place the piece
*******************************************************************/
void draw_wanted_square(int x,int y)
	{
	extern Window gridwindow;
	int i,j;
	for(i=0;i<8;i++)
		for(j=0;j<8;j++)
			{
			draw_pixel(x*GRID_SIZE+ i*3,
				   y * GRID_SIZE + j * 3,
				   gridwindow,6);
			}
	}
/*******************************************************************
void draw_grid(void)
draw the grid - parts of grid that we can place a piece on are shaded
*******************************************************************/

void draw_grid(void)
	{
	extern struct piece_rec piece_array[6];
	extern struct play_grid grid;
	extern Display * display;
	extern Window gridwindow;
	int i,j;
	XClearWindow(display,gridwindow);
	for (i=0;i<MAXSIZE;i++)
		for (j=0;j<MAXSIZE;j++)
			if (grid.matrix[i][j].wanted)
				{
				draw_wanted_square(i,j); 
				}
			else
				{
			
				}
	/* if any piece on grid then draw it there */
	for (i=1;i<6;i++)
		if (piece_array[i].in_use)
                	display_piece_on_grid(i);

	}
/*******************************************************************
void grid_init(void)
initialise the grid window
*******************************************************************/

void grid_init(void)
	{
	extern Window gridwindow,main_window;
	extern unsigned long foreground,background;
	extern Display * display;
	gridwindow =XCreateSimpleWindow(display,main_window,
					GRID_X_LOC,
					GRID_Y_LOC,
					(GRID_SIZE*MAXSIZE),
					(GRID_SIZE*MAXSIZE),
					2,foreground,background);
	XMapWindow(display,gridwindow);
	XSelectInput(display,gridwindow,ButtonPressMask | ExposureMask);
	get_grid("grid1");
	}
/*******************************************************************
void remove_piece(int piece_number)
 remove a piece from the playing grid
*******************************************************************/
void remove_piece(int piece_number)
	{
	extern struct play_grid grid;
	int i,j;
	for (i=0;i<MAXSIZE;i++)
		for (j=0;j<MAXSIZE;j++)
			if (grid.matrix[i][j].filled_by == piece_number)
				grid.matrix[i][j].filled_by = 0;

	}
/*******************************************************************
void update_grid(XEvent event)
decides whether to place a piece on the grid (or attempts to) or
if we clicked on another piece then lift it off the grid and make
it active
*******************************************************************/
void update_grid(XEvent event)
	{
	extern Display *display;
	extern struct play_grid grid;
	extern int active_piece;
	extern struct piece_rec piece_array[6];
	int mousex,mousey;
        int newactive;
	/*get coords of where they clicked */
	/* round to grid location mousex,mousey*/
	mousex =  event.xbutton.x/GRID_SIZE;
        mousey =  event.xbutton.y/GRID_SIZE;
	if (newactive = grid.matrix[mousex][mousey].filled_by)
		{
		/* remove piece from board */
		piece_array[newactive].in_use = false;
		remove_piece(newactive);
		/* make it the active piece */
		active_piece = newactive;
		/* show piece is usable */
		display_piece_on_piecewin(active_piece);
		/* redraw grid */
		draw_grid();
                }
	else
        	{
		/* place piece */
		/* need to modify mousex & mousey to account for
		the changing location of the top left hand corner in
		relation to the pieces location bought around by rotation/mirroring
		so user can know where piece will land
		 3     |  0
		90-180 | 0 -- 90 deg      the quadrants
		----------------
	       180-270 | 270-360 deg
		 2     |  1
		*/
		if (piece_array[active_piece].block[1].x > 0)
			if (piece_array[active_piece].block[1].y > 0)
				{
				/* 0 blocks in 0-90 quadrant*/
				mousex = mousex + piece_array[active_piece].offset[0].x;
				mousey = mousey + piece_array[active_piece].offset[0].y;
				}
			else
				{
				/* 3 blocks in 90-180 quadrant*/
				mousex = mousex + piece_array[active_piece].offset[3].x;
				mousey = mousey + piece_array[active_piece].offset[3].y;
				}
				
		else
			if (piece_array[active_piece].block[1].y > 0)
				{
				/*1 blocks in 270-360 quad*/
				mousex = mousex + piece_array[active_piece].offset[1].x;
				mousey = mousey + piece_array[active_piece].offset[1].y;
				}
			else
				{
				/*2 blocks in 180-270 quad*/
				mousex = mousex + piece_array[active_piece].offset[2].x;
				mousey = mousey + piece_array[active_piece].offset[2].y;
				}
		/* try and place piece */
		if (can_place(mousex,mousey))
			place_here(mousex,mousey);
		else
			{
			XBell(display,30);
			}
		}
	}
